# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2001,2002,2003 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# $Id: _authset.awk,v 1.1 2004/08/08 21:15:38 carlo Exp $
# =====================================================================
# void _authset(string auth, int type [,string file])
#
# Handle basic HTTP authorization.
# =====================================================================

function _authset(s,t,ofile,		a,S,err) {

   if (ofile == "") ofile = _STDOUT

   if (s == "") return				# ignore null requests.

   S = s					# Copy to workarea.

   if (!sub(/^[Bb][Aa][Ss][Ii][Cc] +/,"",S)) err=1	# Basic auth only
   if (S !~ /^[a-zA-Z0-9\/+]+=*[\r]?$/) err=1		# Base64 only

   if (err) {
      _authclear(ofile)
      return
   }

   split(_b64decode(S),a,":")

   # For user-id and password to be valid, they must contain only
   # allowed characters. Note that the password field may be null,
   # and that this is acceptable.

   if (a[1] ~ /^[-_a-zA-Z0-9@.]+$/ && \
			a[2] ~ /^[-+_a-zA-Z0-9@.!]*$/) {

      printf("CSA_AUTH_USER='%s'\n", a[1]) >> ofile
      _rcassign("CSA_AUTH_PW", a[2], ofile)

      # The application-level token must be passed to the application,
      # to be re-sent to the client with every response if necessary.
      # Note the re-use of the same variable that would be set by the
      # server in case of HTTP authorization.

      printf("HTTP_AUTHORIZATION='%s'\n", s) >> ofile
   }

   else {
      # If anything is wrong, clear all auth vars.
      _authclear(ofile)
   }

   # CSA_AUTH_TYPE needs to be set *anyway*, but it may be overridden
   # by server-based auth, further down.

   if (t == 0) print "CSA_AUTH_TYPE=basic" >> ofile
   else print "CSA_AUTH_TYPE=custom" >> ofile

   # Server-based authentication, if used, *MUST* override the one
   # attained by all other means, and the application-level token
   # must be cleared. If the server token does not match the allowed
   # pattern, then it is cleared too.

   if (ENVIRON["REMOTE_USER"] ~ /^[-_a-zA-Z0-9@.]+$/) {

      printf("CSA_AUTH_USER='%s'\n", ENVIRON["REMOTE_USER"]) >> ofile
      print "CSA_AUTH_OK=1;CSA_AUTH_PW=();HTTP_AUTHORIZATION=()" >> ofile
      print "CSA_AUTH_TYPE=server" >> ofile
   }
   else print "REMOTE_USER=()" >> ofile
}

