# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2001,2002,2003,2004 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# $Id: _uncgi.awk,v 1.1 2004/08/08 21:15:38 carlo Exp $

function _uncgi(s,o_file,auth,rdfmap,		a,b,d,i,j,k,n) {

   # rdfmap is "0" for "scalar", "1" for RwA resource, "2" for URL.

   if (s == "") return			# Nothing to do.

   if (o_file == "") o_file = _STDOUT	# set default

   # If input data contains assignments it is truncated to 10 KB to
   # prevent buffer and environment overflows in the calling program.
   # Actually, the input is coming from both stdin and QUERY_STRING, so
   # this check should be improved to account for both, to become really
   # effective.
   # On the contrary, if input does not contain assignments I consider
   # it an ISINDEX query and assign its value to the CSA_ISINDEX
   # variable. In this case I truncate the value to 512 bytes, although
   # it should already have been truncated to something shorter by the
   # Web server itself.

   if (s ~ /=/) {
      if (ENVIRON["CONTENT_LENGTH"] > rpc_maxsize)
	 return _EMSGSIZE		# Max. input data size exceeded.
   }
   else {
      s = "CSA_ISINDEX=" substr(s,1,512)
      gsub(/&/, "%26", s)
   }

   # remove trailing assignments that have been invalidated by
   # the truncation process
   sub(/&[^&=]*$/, "", s)
   sub(/%.?$/, "", s)

   # single-quotes must be doubled for rc(1),
   # whether they are URI-escaped or not.
   ###gsub(/'/, "''", s)
   ###gsub(/%27/, "''", s)

   # split-up the input assignments
   if ((n = split(s, a, "\\&")) == 0) return

   # Note: a construct like "for (i in a)" may not preserve ordering.
   while (++i <= n) {

       # split each "name=value" pair
       split(a[i],b,"=")

       # Handle virtual headers.
       if (b[1] ~ /^[Xx]-/) {
	  gsub(/-/, "_", b[1])
	  b[1] = toupper(b[1])
       }

       # normalize white-space and line-breaks a-la uncgi,
       # and remove leading "." from names. Strip characters
       # that would make textarea names look like virtual headers.
       # Unlike uncgi, for a variable to be normalized it must
       # begin with one dot.

       if (sub(/^\.([Xx]-)?/, "", b[1])) {
	  sub(/^[ \t]*/,"",b[1])
	  sub(/[ \t]*$/,"",b[1])
	  gsub(/%0D%0A/, "%0A", b[2])
	  gsub(/%0D/, "%0A", b[2])
       }

       # replace invalid characters with underscores (_) in variable names

       gsub(/[^A-Za-z0-9_]/, "_", b[1])

       # skip variable names that begin with a number followed by  
       # an underscore, as they are reserved for CSA internal use
       # (for instance to generate the file-name associated with
       # a file-type argument).

       if (b[1] ~ /^[0-9]+_/) continue

       # identify escape codes and perform order-independent conversions
       j = split(b[2],d,"%")
       b[2] = d[1]
       k=1
       while(++k <= j) {
	 key = substr(d[k],1,2)
	 repl = _CHARMAP[key]

	 # restore original code if lookup failed
	 if (repl == "") repl = "%" key

	 b[2] = b[2] repl substr(d[k],3)
       }

       # perform order-dependent conversions
       gsub(/\+/, " ", b[2])
       gsub(/%2[Bb]/, "+", b[2])
       gsub(/%25/, "%", b[2])
       gsub(/%0[Aa]/, "\n", b[2])

       # convert selected chars purposedly not listed in _CHARMAP[].
       gsub(/%27/,"'",b[2])
       gsub(/%0[Dd]/,"\015",b[2])

       # extract HTTP authorization if requested. To cope with how
       # HTTP cookies are handled by browsers, in case of multiple
       # assignments only the first one is retained here.

       if (toupper(b[1]) == "AUTHORIZATION") {
	  if (auth) _authset(b[2],1,o_file)
	  auth = 0		 # ignore any further re-assignments.
       }

       else _cgivar(b[1], b[2], rdfmap)		# do the assignment
   }
}

